#pragma once

#include <ts/defs.hpp>

// Support for scenario functions
#include <ts/scenario/functors/functions.hpp>
// Test scenario with C++ functors
#include <ts/scenario/scenario.hpp>
// ScenarioBase class
#include <ts/scenario/base.hpp>

/** @def CPPTESK_SCENARIO_EX
 * Define scenario class as a child of another scenario class.
 * @note Broken. Do not use.
 * @param className name of the new scenario class.
 * @param parentName name of the parent scenario class.
 */
#ifdef CPPTESK_SCENARIO_NON_VIRTUAL_BASE
    #define CPPTESK_SCENARIO_EX(className, parentName) \
    class className : public parentName \
    , public virtual cpptesk::utils::DerivedType<className, parentName>
#else
    #define CPPTESK_SCENARIO_EX(className, parentName) \
    class className : public virtual parentName \
    , public virtual cpptesk::utils::DerivedType<className, parentName>
#endif

/** @def CPPTESK_SCENARIO
 * Define scenario class.
 * The new scenario class is implicitly derived from cpptesk::ts::ScenarioBase.
 * @param className name of the new scenario class.
 */
#define CPPTESK_SCENARIO(className) CPPTESK_SCENARIO_EX(className, cpptesk::ts::ScenarioBase)

/** @def CPPTESK_SCENARIO_CTOR
 * Define default scenario class constructor.
 * The constructor calls default constructor of the parent scenario class scenario class, if any, or cpptesk::ts::ScenarioBase().
 * @note Broken. Do not use.
 * @param className name of the scenario class.
 */
#define CPPTESK_SCENARIO_CTOR(className) className() : parent_class_t()

/** @def CPPTESK_ADD_SCENARIO_METHOD
 * Register the given scenario method from the given scenario class.
 */
#define CPPTESK_ADD_SCENARIO_METHOD(scenario_method) \
    add_scenario_method(#scenario_method, &scenario_method)

#if (defined(CPPTESK_SHORT_NAMES) || defined(CPPTESK_SCENARIO_SHORT_NAMES))\
    && !defined(CPPTESK_SCENARIO_NO_SHORT_NAMES)
#define SCENARIO CPPTESK_SCENARIO
#define SCENARIO_EX CPPTESK_SCENARIO_EX
#define SCENARIO_CTOR CPPTESK_SCENARIO_CTOR
#define ADD_SCENARIO_METHOD CPPTESK_ADD_SCENARIO_METHOD
#endif

#if (defined(CPPTESK_SHORT_SHORT_NAMES) || defined(CPPTESK_SCENARIO_SHORT_SHORT_NAMES))\
    && !defined(CPPTESK_SCENARIO_NO_SHORT_NAMES)
#define SCENARIO CPPTESK_SCENARIO
#define SCENARIO_EX CPPTESK_SCENARIO_EX
#define SCENARIO_CTOR CPPTESK_SCENARIO_CTOR
#define SCENARIO_METHOD CPPTESK_ADD_SCENARIO_METHOD
#endif
