
#include "account_scen.h"

using namespace cpptesk::ts;

template<typename C>
void print_coverage(cpptesk::tracer::coverage::CoverageTracker& ct) {
	std::cout << "Coverage " << C::name() << std::endl;
	for(typename C::ElementIterator i = C::begin(); i != C::end(); ++i) {
		std::cout << "  " << (*i).label << " : " << ct[C((*i).id)] << std::endl;
	}
}

#ifdef NETFSM
#include <netfsm/engines.hpp>
#include <utils/testreg.h>
extern "C" { int start_scenario_with_wrapper(void (*wrapper)(void)) { return 0; } }
Account impl(-3);
AccountScenarioEx scen_rnd(3, &impl);
AccountScenarioEx scen_fsm(3, &impl);
TEST_REGISTRY_BEGIN
REGISTER_TEST(scen_rnd, engine::rnd, "-nt")
REGISTER_TEST(scen_fsm, engine::fsm, "-nt")
TEST_REGISTRY_END
#else

#ifdef NDFSM
        #include <ts/ndfsm.hpp>
#else
         #include <ts/dfsm.hpp>
#endif

#endif


CPPTESK_DEFINE_ALIAS_COVERAGE(Alias_Cov, "Alias", AccountModel::DepositStart);


int main(int argc, char **argv) {
#ifdef NETFSM
        char name[15] = "scen_rnd";
        if(argc > 1)
        {
            strncpy(name, argv[1], 15);
        }

        startScenario_TestRegistry(name, LOCAL);
#else
	Account impl(-3 /*  */ );

	AccountScenarioEx scenario(3 /*  */, &impl);

#ifdef NDFSM
	start_ndfsm(argc, argv, scenario.getTestScenario());
#else
	start_dfsm(argc, argv, scenario.getTestScenario());
#endif
	std::cout<< "------------------" << std::endl;
	print_coverage<AccountModel::DepositStart>(scenario.mediator.coverage);
	std::cout<< "------------------" << std::endl;
	print_coverage<AccountModel::DepositEnd>(scenario.mediator.coverage);
	std::cout<< "------------------" << std::endl;
	print_coverage<AccountModel::Deposit>(scenario.mediator.coverage);
	std::cout<< "------------------" << std::endl;
	print_coverage<AccountModel::WithdrawStart>(scenario.mediator.coverage);
	std::cout<< "------------------" << std::endl;
	print_coverage<AccountModel::WithdrawEnd>(scenario.mediator.coverage);
	std::cout<< "------------------" << std::endl;
	print_coverage<AccountModel::Withdraw>(scenario.mediator.coverage);
	std::cout<< "------------------" << std::endl;
	AccountModel::DepositStart x = AccountModel::DepositStart::NEGATIVE;
	Alias_Cov y = x;
	Alias_Cov z = x.id;
#endif
	return 0;
}
