/** @file fifo_scen.cpp
 *   FIFO.
 *
 * @author   (chupilko@ispras.ru)
 */

#include <fifo_scen.h>

namespace cpptesk {
namespace examples {
namespace fifo {

bool FIFOScenario::init(int argc, char ** argv)
{
    return true;
}

void FIFOScenario::finish()
{
    //  :   run  0.
    dut.finalize();
}

bool FIFOScenario::scen_nop(Ctx& ctx)
{
    //    (    26 ):
    // int& param1 = IVAR(a);
    // int& param2 = IVAR(b);

    //        IBEGIN

    //      IBEGIN
    IBEGIN
    //  IBEGIN   
    // for(param1 = 0; param1 < 2; param1++) {
    //    for(param2 = 0; param2 < 2; param2++) {

    //     IACTION    .
    IACTION
    {
        //  
        dut.refresh_coverage();

        //     1 .
        dut.cycle();

        //     .
        YIELD(dut.verdict());
    }
    //     }
    // }
    IEND
}

bool FIFOScenario::scen_push(Ctx& ctx)
{
    int& tag = IVAR(a);

    IBEGIN
    for(tag = 0; tag < 32; tag++) {
        IACTION
        {
            //   :         .
            if(dut.isIface1Ready() && !dut.is_full()) {
                //   ,          .
                //        .
                InputData msg = InputData();
                int data = msg.get_data();
                msg.set_data(data & ~0xF8 | tag);
                CPPTESK_CALL_STIMULUS_OF(dut, FIFO::push_msg, dut.iface1, msg);
            }
            YIELD(dut.verdict());
        }
    }
    IEND
}

bool FIFOScenario::scen_pop(cpptesk::ts::IntAbcCtx& ctx)
{
    IBEGIN
    IACTION
    {
        //   :         .
        if(dut.isIface2Ready() && !dut.is_empty()) {
            //   ,          .
            //        .
            InputData msg = InputData();
            CPPTESK_CALL_STIMULUS_OF(dut, FIFO::pop_msg, dut.iface2, msg);
        }
        YIELD(dut.verdict());
    }
    IEND
}

std::string FIFOScenario::get_state_fifo()
{
    std::string state;
    std::stringstream out;

    out << "{fifo:" << dut.fifo_fulness() << ", write_started:" << dut.isIface1Ready() << ", read_started:" << dut.isIface2Ready() << "}";
    state = out.str();
#ifdef DEBUG
    std::cout << "get_state_fifo(): " << state << std::endl;
#endif
    return state;
}

FIFOScenario::FIFOScenario(void) : ScenarioBase()
{
}
FIFOScenario::~FIFOScenario(void) {}

ExampleScenario::ExampleScenario(void) : FIFOScenario()
{
    //   .    : ,     .
    setup("FIFOExtScenario", &FIFOScenario::init, &FIFOScenario::finish, &FIFOScenario::get_state_fifo);

    //   . NOP     .
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_nop);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_push);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_pop);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

