/** @file multififo_media.h
 * Header for the mediator of united FIFO's model.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <hw/veritool/media.hpp>
#include <hw/media.hpp>

#include <multififo_model.h>
#include <interface.h>
#include <fifo_interface.h>

#include <string>

namespace cpptesk {
namespace examples {
namespace fifo {

/**
 * @brief Mediator for Multififo model.
 */
CPPTESK_VERITOOL_MEDIA(MultififoMediator, Multififo)
{
public:
    MultififoMediator();
    virtual ~MultififoMediator();

    /// Max delay time of registered reactions (in cycles).
    const static int REACTION_TIMEOUT = 100;

    virtual bool isIface1aReady() const;
    virtual bool isIface1bReady() const;
    virtual bool isIface2Ready() const;

    /// Input interface iface1a serializer.
    CPPTESK_DECLARE_PROCESS(serialize_iface1a);
    /// Input interface iface1b serializer.
    CPPTESK_DECLARE_PROCESS(serialize_iface1b);
    /// Input interface iface2 serializer.
    CPPTESK_DECLARE_PROCESS(serialize_iface2);
//    CPPTESK_DECLARE_TWO_LEVEL_ARBITER(CPPTESK_FIFO_ARBITER, CPPTESK_COMPARE_ARBITER, matcher_iface3);

    /// Output interface iface3 deserializer.
    CPPTESK_DECLARE_PROCESS(deserialize_iface3);

    CPPTESK_DECLARE_PROCESS(deserialize_inner_iface0);
    CPPTESK_DECLARE_PROCESS(deserialize_inner_iface1);

    virtual void cycle();
};

}}}
