/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Field Metric for rules
///////////////////////////////////////////////////////////////////////////////

#include <hw/diagnostics/metrics/field_metric.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {
    
int FieldMetric::compare(const cpptesk::hw::Message &m1, const cpptesk::hw::Message &m2)
{
    return fields_metric(m1, m2);
}

FieldMetric* FieldMetric::getMetric()
{
    return cpptesk::hw::diagnostics::SingleMetric<FieldMetric>::getInstance();
}

/// Return number of identical fields
int FieldMetric::fields_metric(const cpptesk::hw::Message &m1, const cpptesk::hw::Message &m2)
{
    int result = FALSE_MATCH;

    if(m1.size() == m2.size())
    {
        if(m1.size() == 0)
        {
            result = EXACT_MATCH;
        }
        else
        {
            uint closeness = 0; // closeness between two messages
            uint comparable_fields = 0;
            
            for(cpptesk::hw::Message::const_iterator i = m1.begin(),
                                                     j = m2.begin();
                i != m1.end(); i++, j++)
            {
                const Field<> &field_from_m1 = m1.**i;
                const Field<> &field_from_m2 = m2.**j;

                if(field_from_m1.isComparable())
                {
                    if(field_from_m1.equals(field_from_m2))
                        closeness++;
                }
                else
                    comparable_fields++;
            }
            result = (WORST_MATCH * (m1.size() - comparable_fields - closeness)) / (m1.size() - comparable_fields);
        }
    }
    return result;
}
    
}}}
