/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Hamming Metric for diagnostics rules
///////////////////////////////////////////////////////////////////////////////

#pragma once
#include <hw/diagnostics/metrics/metric.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

/// Return number of identical bits
class HammingMetric: public BaseMetric
{
    friend class SingleMetric<HammingMetric>;

private:
    /// Constructor for HammingMetric class
    HammingMetric() {}
    /// Destructor for HammingMetric class
    virtual ~HammingMetric() {}
    
public:
    /// Use this metric
    int compare(const cpptesk::hw::Message &m1, const cpptesk::hw::Message &m2);

    /// Get HammingMetric object
    static HammingMetric* getMetric();

    /// Return Hamming size
    static int hamming_size(int a, int b); 
    
    /// Return metric synopsis
    std::string toString() {return "Hamming Metric";}    
    std::string operator<< (HammingMetric &_metric) {return _metric.toString();}
};

}}}
