/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Vector of Reaction Pairs
///////////////////////////////////////////////////////////////////////////////

#include <hw/diagnostics/reaction_pair_vector.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

ReactionPairVector::ReactionPairVector() {}
ReactionPairVector::~ReactionPairVector() {}

int ReactionPairVector::size()
{
    return _reaction_pairs.size();
}

bool ReactionPairVector::is_empty()
{
    if(_reaction_pairs.size() == 0)
        return true;

    for(uint i = 0; i < _reaction_pairs.size(); i++)
    {
        Reaction &reaction = _reaction_pairs[i].get_collapsed_reaction();

        if(!(reaction.getError() == MISSING_REACTION && reaction.isExpected()))
            return false;
    }

    return true;
}

//inline void ReactionPairVector::clear() { _reaction_pairs.clear(); }

ReactionPair& ReactionPairVector::operator[] (int index)
{
    return _reaction_pairs[index];
}

void ReactionPairVector::add_reaction_pair(ReactionPair _reaction_pair)
{
    _reaction_pairs.push_back(_reaction_pair);
}

}}} //cpptesk::hw::diagnostics
