/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: ReactionUnit class keeps Reaction and IDs to its predecessors
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/event.hpp>
#include <hw/debug.hpp>

#include <hw/diagnostics/types.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class ReactionUnit
{
public:
    ReactionUnit();
    ReactionUnit(const ReactionUnit &ru);
    ReactionUnit(const cpptesk::hw::Reaction &r);
    ReactionUnit(const cpptesk::hw::Reaction &r, int parent_spec_id, int parent_impl_id);
    virtual ~ReactionUnit();

    /// Collapsed flag
    void set_collapsed();
    bool is_collapsed();

    /// Rule of reaction unit
    void set_collapse_rule(int rule);
    int get_collapse_rule();

    void set_collapse_type(int collapse_type);
    int get_collapse_type();

    void set_id(int id);
    int get_id();

    /// Reaction parent from specification
    void set_parent_spec_id(int parent_id);
    int get_parent_spec_id();

    /// Reaction parent from implementation
    void set_parent_impl_id(int parent_id);
    int get_parent_impl_id();

    typedef cpptesk::hw::debug_level_t dbg_level_t;
    typedef cpptesk::hw::debug_color_t dbg_color_t;

    void print_description(dbg_level_t dbg_level, dbg_color_t fg_level, dbg_color_t bg_level);
    void print_description(dbg_level_t dbg_level);
    void print_description();
    void print_short_description(dbg_level_t dbg_level, dbg_color_t fg_level, dbg_color_t bg_level, bool spec);
    void print_short_description(dbg_level_t dbg_level, bool spec);
    std::string toString();
    std::string operator<< (ReactionUnit &ru);

    cpptesk::hw::Reaction& getReaction();

protected:
    /// Collapsed flag
    bool _collapsed;
    
    int  _parent_spec_id, _parent_impl_id;
    int  _id;
    int  _collapse_rule;
    int  _collapse_type;
    
    cpptesk::hw::Reaction _reaction;
};

}}}
