/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Vector of Reaction Units
///////////////////////////////////////////////////////////////////////////////

#include <hw/diagnostics/reaction_unit_vector.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

ReactionUnitVector::ReactionUnitVector() {}
ReactionUnitVector::~ReactionUnitVector() {}

ReactionUnitVector::ReactionUnitVector(cpptesk::hw::Reaction &r1, cpptesk::hw::Reaction &r2,
                         int parent_r1_spec_id, int parent_r1_impl_id,
                         int parent_r2_spec_id, int parent_r2_impl_id)
{
    add_reaction(r1, parent_r1_spec_id, parent_r1_impl_id);
    add_reaction(r2, parent_r2_spec_id, parent_r2_impl_id);
}

ReactionUnitVector::ReactionUnitVector(cpptesk::hw::Reaction &r,
                         int parent_spec_id, int parent_impl_id)
{
    add_reaction(r, parent_spec_id, parent_impl_id);
}

ReactionUnit& ReactionUnitVector::operator[] (uint index)
{
    return _reaction_vector[index];
}

uint ReactionUnitVector::size() const
{
    return _reaction_vector.size();
}

bool ReactionUnitVector::is_empty() const
{
    return size() == 0;
}

void ReactionUnitVector::add_reaction_unit(ReactionUnit ru)
{
    ru.set_id(_reaction_vector.size());
    _reaction_vector.push_back(ru);
}

void ReactionUnitVector::add_reaction(const cpptesk::hw::Reaction &reaction, int parent_spec, int parent_impl)
{
    ReactionUnit ru(reaction, parent_spec, parent_impl);
    add_reaction_unit(ru);
}

void ReactionUnitVector::add_reaction(const cpptesk::hw::Reaction &reaction)
{
    add_reaction(reaction, -1, -1);
}

void ReactionUnitVector::add_reaction_units(ReactionUnitVector &rs)
{
    for(uint i = 0; i < rs.size(); i++)
    {
        add_reaction((rs[i]).getReaction(), (rs[i]).get_parent_spec_id(), (rs[i]).get_parent_impl_id());
    }
}

void ReactionUnitVector::init(const event_list_t &reactions)
{
    for(event_list_t::const_iterator i = reactions.begin(); i != reactions.end(); i++)
    {
       const Reaction *reaction = dynamic_cast<const Reaction*>(i->get());
       add_reaction(*reaction);
    }
}

}}} // cpptesk::hw::diagnostics
