/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Base abstract class for rules
///////////////////////////////////////////////////////////////////////////////

#include <hw/diagnostics/rules/rule.hpp>
#include <hw/diagnostics/error_model.hpp>
//#include "rule.hpp"

namespace cpptesk {
namespace hw {
namespace diagnostics {

void Rule::assign_rule_number(ReactionUnit &ru1, ReactionUnit &ru2,
                              int rule, CollapseType type)
{
    ru1.set_collapse_rule(rule);
    ru1.set_collapse_type(type);

    
    if(rule != BASIC_RULE)
    {
        ru2.set_collapse_rule(rule);
        ru2.set_collapse_type(type);
    }
};

int Rule::compare(const cpptesk::hw::Message &m1, const cpptesk::hw::Message &m2)
{
    // Use only one metric
    int result = cpptesk::hw::diagnostics::BaseMetric::metric->compare(m1, m2);
    
    return result;
}

int Rule::compare(const cpptesk::hw::Message *m1, const cpptesk::hw::Message *m2)
{
    return compare(*m1, *m2);
}

std::string Rule::operator<< (Rule &rule)
{
    return rule.toString();
}

}}}
