/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Printer for Diagnostics Results
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <fstream>

#include <hw/event.hpp>

#include <hw/report/model.hpp>
#include <hw/report/report_printer.hpp>

namespace cpptesk {
namespace hw {
namespace report {

class ConsolePrinter: public ReportPrinter
{
public:
    ConsolePrinter();
    virtual ~ConsolePrinter();

    virtual void print(std::ostream &out, const Report &report);
    
    void print(std::ostream &out, const Report &report, bool debug_color);
       
private:
    /// Prints the results of the test
    
    /// Max width of line
    int _line_width;
    
    /// Enabling colors
    int _debug_color;
    
    /// Precision of double values
    unsigned _frequency_precision;

    /// Print failures from report
    void _print_failures(std::ostream &out, const cpptesk::hw::report::Failures &failures);
    /// Print DATA spec/impl message
    void _data_line(std::ostream &out, const cpptesk::hw::report::Failure &failure, const int &first_column_width, const bool &impl);
    /// Print spec DATA message (_data_line(impl = false))
    void _data_spec(std::ostream &out, const cpptesk::hw::report::Failure &failure, const int &first_column_width);
    /// Print impl DATA message (_data_line(impl = true))
    void _data_impl(std::ostream &out, const cpptesk::hw::report::Failure &failure, const int &first_column_width);

    /// Print time of failure
    void _time_failure(std::ostream &out, const cpptesk::hw::report::Failure &failure, const int &first_column_width);
    /// Print iface of failure
    void _iface_failure(std::ostream &out, const cpptesk::hw::report::Failure &failure, const int &first_column_width);
    /// Print title of failure
    void _title_failures(std::ostream &out, const cpptesk::hw::report::Failure &failure, const int &first_column_width);
    
    /// Print statistic part of report
    void _print_statistic(std::ostream &out, const cpptesk::hw::report::Statistics &statistics);
    /// Print simulation part of report
    void _print_simulation(std::ostream &out, const cpptesk::hw::report::Simulation &simulation);

    /// Print explanation only for spec reaction
    void _print_explanation(std::ostream &out, std::string explanation);

    /// Print message with color designation
    //  Printing field_list message
    void _print_color_field(std::ostream &out, const cpptesk::hw::report::FieldList &field_list, const debug_color_t color);

    /// Return size of string of digital value
    int _get_size(int temp_num);
    int _get_size(double temp_num, unsigned precision);

    /// Print separation line
    void _print_line(std::ostream &out, int length);
    void _print_line(std::ostream &out);

    /// Add symbol to string
    void _add_symbol_top(std::string &str, char* symbol, const int iterator);
    void _add_symbol(std::string &str, char* symbol, const int iterator);

    /// Return max width of strings
    int get_max_width(const std::string &str_0, const std::string &str_1, const std::string &str_2);
};

}}}
