/** @file counter_media.cpp
 * This file contains implementation of the counter model mediator class.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <counter_media.h>

#include <sync.h>

#include <interface.h>
#include <vpi_media.h>

#include <iostream>

namespace cpptesk {
namespace examples {
namespace counter {

CounterMediator::CounterMediator()
{
    SET_DEBUG_LEVEL(cpptesk::hw::DEBUG_INFO, true);
    SET_DEBUG_STYLE(cpptesk::hw::DEBUG_USER, cpptesk::hw::BLACK, cpptesk::hw::CYAN, cpptesk::hw::CYAN, cpptesk::hw::BLACK);

    SET_INPUT_ADAPTER(iface1, CounterMediator::serialize_iface1);
    SET_OUTPUT_ADAPTER_NO_LISTENER(iface2, CounterMediator::deserialize_iface2);

    SET_REACTION_TIMEOUT(CounterMediator::REACTION_TIMEOUT);
}

CounterMediator::~CounterMediator() {}

bool CounterMediator::isIface1Ready() const
{
    return Counter::isIface1Ready();
}

DEFINE_PROCESS(CounterMediator::serialize_iface1)
{
    InputData msg = CAST_MESSAGE(InputData);

    START_PROCESS();

    DEBUG_PRINT(cpptesk::hw::DEBUG_USER, "CounterMediator::serialize_iface1");
    DEBUG_PRINT(cpptesk::hw::DEBUG_USER, "CounterMediator::serialize_iface1: message=" << GET_MESSAGE());

    CAPTURE_IFACE();

    inputs.inc = 1;

    RELEASE_IFACE();

    CYCLE();

    STOP_PROCESS();
}

DEFINE_PROCESS(CounterMediator::deserialize_iface2)
{
    OutputData &msg = CAST_MESSAGE(OutputData);

    START_PROCESS();

    DEBUG_PRINT_FILE_LINE(cpptesk::hw::DEBUG_USER, "CounterMediator::deserialize_iface2");

    WAIT_REACTION(true);

    msg.set_data(outputs.data);

    NEXT_REACTION();

    STOP_PROCESS();
}

}}}

