/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef RAND_LFG_H_INCLUDED
#define RAND_LFG_H_INCLUDED

#include <stdint.h>

#define RAND_LFG_J    24
#define RAND_LFG_K    55
#define RAND_LFG_SEED 0xDEADBEAFull

///////////////////////////////////////////////////////////////////////////////
// Additive lagged Fibonacci random number generator
///////////////////////////////////////////////////////////////////////////////

typedef struct LFG
{
    // Index n-j.
    uint32_t j;
    // Index n-k.
    uint32_t k;
    // Previously generated numbers.
    uint32_t state[RAND_LFG_K];   
} LFG;

/** @fn void init_LFG(LFG *generator)
 * Initializes the lagged Fibonacci generator.
 * @param generator the random number generator.
 */
void init_LFG(LFG *generator);

/** @fn void seed_LFG(LFG *generator, uint32_t seed)
 * Sets the seed of the lagged Fibonacci generator.
 * @param generator the random number generator.
 * @param seed the seed.
 */
void seed_LFG(LFG *generator, uint32_t seed);

/** @fn uint32_t next_LFG(LFG *generator)
 * Returns the next number generated by the lagged Fibonacci generator.
 * @param generator the random number generator.
 */
uint32_t next_LFG(LFG *generator);

///////////////////////////////////////////////////////////////////////////////
// Modified additive lagged Fibonacci random number generator
///////////////////////////////////////////////////////////////////////////////

typedef struct MLFG
{
    LFG x;
    LFG y;
} MLFG;

/** @fn void init_MLFG(MLFG *generator)
 * Initializes the modified lagged Fibonacci generator.
 * @param generator the random number generator.
 */
void init_MLFG(MLFG *generator);

/** @fn void seed_MLFG(MLFG *generator, uint32_t seed)
 * Sets the seed of the modified lagged Fibonacci generator.
 * @param generator the random number generator.
 * @param seed the seed.
 */
void seed_MLFG(MLFG *generator, uint32_t seed);

/** @fn uint32_t next_MLFG(MLFG *generator)
 * Returns the next number generated by the modified lagged Fibonacci generator.
 * @param generator the random number generator.
 */
uint32_t next_MLFG(MLFG *generator);

#endif // RAND_LFG_H_INCLUDED
