/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>

#include <generator/exception.h>
#include <generator/tracer.h>

extern char* ts_print_backtrace(char* prefix);

void netfsm_default_assertion_handler(const char* format, va_list arg_list)
{
    char buff[1024];
    vsprintf(buff, format, arg_list);
    traceInternalError_TestEngine("ASSERTION", buff);
    closeTrace_TestEngine();
    exit(1);
}

#if defined(__GNUC__) && !defined(_WIN32)

void netfsm_default_exception_handler(siginfo_t* info, void* context)
{
#ifndef __CYGWIN__
  char* backtrace_info;
#endif
  static char buff[1024];   
  
  fire_exception_event();

  switch (info->si_signo)
   {
    case SIGINT :
      traceSystemInfo_TestEngine("SIGNAL" ,"Test has been terminated by signal SIGINT");
      closeTrace_TestEngine();
      break;
    case SIGSEGV :
      #if !defined(SEGV_MAPERR) || !defined(SEGV_ACCERR)
        sprintf( buff, "Test has been terminated by signal SIGSEGV at address '%p'", info->si_addr );
      #else
        if (info->si_code == SEGV_MAPERR)
          sprintf( buff, "Invalid memory address '%p'", info->si_addr );
        else if (info->si_code == SEGV_ACCERR)
          sprintf( buff, "Invalid permissions for memory at '%p'", info->si_addr );
        else
          sprintf( buff, "Unknown memory access violation at '%p'", info->si_addr );
      #endif
      
#ifndef __CYGWIN__
      backtrace_info = ts_print_backtrace(buff);
      if (backtrace_info != NULL)
      {
          traceInternalError_TestEngine("SIGNAL", backtrace_info);
          free(backtrace_info);
      }
      else
      {
          traceInternalError_TestEngine("SIGNAL", buff);
      }
#else
          traceInternalError_TestEngine("SIGNAL", buff);
#endif
      
      closeTrace_TestEngine();
      break;
    case SIGTERM :
      traceSystemInfo_TestEngine("SIGNAL", "Test has been terminated by signal SIGTERM");
      closeTrace_TestEngine();
      break;
    default :
      sprintf( buff, "Unexpected signal '%d'", info->si_signo );
      traceInternalError_TestEngine("SIGNAL", buff);
      closeTrace_TestEngine();
      break;
   }
} 

#elif defined(_MSC_VER)

int netfsm_default_exception_handler(LPEXCEPTION_POINTERS info)
{
static char buff[256];

  switch (info->ExceptionRecord->ExceptionCode)
   {
    case EXCEPTION_BREAKPOINT :
    case EXCEPTION_SINGLE_STEP :
      return EXCEPTION_CONTINUE_SEARCH;
    case EXCEPTION_ACCESS_VIOLATION :
      fire_exception_event();
      sprintf( buff, "The instruction at '0x%p' referenced memory at '0x%p'. The memory could not be '%s'.",
                                info->ExceptionRecord->ExceptionAddress,
                                info->ExceptionRecord->ExceptionInformation[1],
                                info->ExceptionRecord->ExceptionInformation[0] ? "written" : "read"
             );
      traceExceptionInfo_TestEngine("signal", buff);
      traceInternalError_TestEngine("signal", "Memory access violation");
      closeTrace_TestEngine();
      break;
    default :
      fire_exception_event();
      sprintf( buff, "Exception 0x%lX occurs at 0x%p",
                                info->ExceptionRecord->ExceptionCode, 
                                info->ExceptionRecord->ExceptionAddress
             );
      traceInternalError_TestEngine("signal", buff);
      closeTrace_TestEngine();
      break;
   }
  return EXCEPTION_CONTINUE_SEARCH;
}

#endif

