/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <sgl/utils.h>

TE_UpdateStatus updateInfo_Edge(TE_GraphInfo *info, SGL_Edge *edge, TE_UpdateType type, SGL_UpdateStatus status)
{
    TE_UpdateStatus result = TE_OLD_ALL;
    
    if((status & SGL_US_NEW_START_NODE) != 0)
    {
        result |= TE_NEW_NODE;
        info->nodes_traversed++;

        if(type == TE_LOCAL)
            { info->nodes_traversed_locally++; }
    }

    if((status & SGL_US_NEW_END_NODE) != 0)
    {
        result |= TE_NEW_NODE;
        info->nodes_traversed++;

        if(type == TE_LOCAL)
            { info->nodes_traversed_locally++; }
    }

    if((status & SGL_US_FIN_START_NODE) != 0)
    {
        info->nodes_finished++;
    }

    if((status & SGL_US_NEW_EDGE) != 0)
    {
        result |= TE_NEW_ARC;

        if(sgl_isNew_Edge(edge))
        {
            info->arcs_known++;
        }
        else
        {
            info->arcs_traversed++;

            if(!sgl_isDeterministic_Edge(edge))
            {
                info->arcs_known++;
                info->arcs_nondeterministic++;
            }

            if((status & SGL_US_NEW_LOOP) != 0)
                { info->loops_traversed++; }
            
            if(type == TE_LOCAL)
                { info->arcs_traversed_locally++; }
        }
    }

    return result;
}

TE_ArcId* createArcId_Edge(SGL_Edge *edge)
{
    SGL_Node *start_node = sgl_getStartNode_Edge(edge);
    SGL_Node *end_node   = sgl_getDegree_Edge(edge) > 0 ? sgl_getEndNode_Edge(edge, 0) : NULL;

    TE_NodeId *start_node_id = sgl_getNodeId_Node(start_node);
    TE_NodeId *end_node_id   = sgl_getNodeId_Node(end_node);

    return create_ArcId(r(start_node_id), edge->stimulus_id, r(end_node_id));
}
