/********** AJAX **********/

var HTTPMCC_VERSION = 2;

function call( request, callback, data )
{
    $(document).ready(function() {
        $.ajax( { type:"GET"
                , async: true
                , url: request
                , data: data
                , dataType: "json"
                , jsonp: "unitesk"
                , success: callback
                }
        );
    });
}

function requestSuitesList( callback )
{
    call( "/suites"
        , callback
        , { version: HTTPMCC_VERSION }
        );
}

function requestSuite( name, callback )
{
    call( "/suite"
        , callback
        , { version: HTTPMCC_VERSION
          , suite: name
          }
        );
}

function requestSuiteSave( name, suite, create, callback )
{
    call( "/suite-save"
        , callback
        , { version: HTTPMCC_VERSION
          , suite: name 
          , json: JSON.stringify( suite )
          , create: true 
          }
        );
}

function requestSuiteDelete( name, callback )
{
    call( "/suite-delete"
        , callback
        , { version: HTTPMCC_VERSION
          , suite: name 
          }
        );
}

function requestTestsList( callback )
{
    call( '/tests'
            , callback
            , { version: HTTPMCC_VERSION }
        );
}

function requestLinksList( callback )
{
    call( '/links'
            , callback
            , { version: HTTPMCC_VERSION }
        );
}

function requestTestrunsList( suite, test, callback )
{
    var p = { version: HTTPMCC_VERSION };
    if( test ) p.test = test;
    if( suite ) p.suite = suite;
    call( "/testruns"
        , callback
        , p
        );
}

function requestNodesList( testrun, callback )
{
    call( '/test'
            , callback
            , { version: HTTPMCC_VERSION
              , testRunId: testrun
              }
        );
}

/***************************/

function mcc_error( title, message )
{
    $.pnotify(
        { pnotify_type: 'error'
        , pnotify_title: title
        , pnotify_text: message
        }
    );
}

var STATUS =
{ OK: " "
, SETTINGS: "Settings Updated"
, SUITE_LOADED: "Suite Loaded"
, STARTING: "Starting"
, STOPPING: "Stopping"
, KILLING:  "Killing"
}

function status( answer, expectedStatus, message )
{
    if( !answer || !answer.status ) return false;
    if( !expectedStatus ) expectedStatus = STATUS.OK;
    
    if( answer.status != expectedStatus )
    {
        inspect( answer );
        alert( message );
        if( !message ) message = answer.status;
        mcc_error( 'Error', message );
        return false;
    }
    return true;
}

var status_style =
{
  NONE:    { ignore: true, result: true }
, INFO:    { ignore: false, style: "severity-info", fade: 500, delay: 2000, result: true }
, WARNING: { ignore: false, style: "severity-warning", fade: 500, delay: 2000, result: true }
, ERROR:   { ignore: false, style: "severity-error", fade: 500, delay: 5000, result: false }
}

function status_old( message )
{
    var style = status_style[message.severity];

    if( !style.ignore )
    {
        var status_elem = $( "#status" );
        status_elem.queue( 
                        function()
                        { 
                            status_elem.addClass( style.style )
                                       .text( message.message ); 
                            $(this).dequeue();
                        } 
                    )
                   .fadeIn( style.fade )
                   .delay( style.delay )
                   .fadeOut( style.fade );
    }
    return style.result;
}

/*********** Row manipulation *************/

function addRow( constructorRow )
{
    var row = constructorRow.clone();
    
    $( ".checkbox-column input", row ).removeAttr( "disabled" );
    $( ".name-column", row ).empty();
    $( "*", row ).removeClass( "hidden" );
    row.removeClass( "constructor-row" ).insertBefore( constructorRow );
    
    return row;
}

function getCheckedRows( table )
{
    return $( "input:checked", table ).parents('tr');
}

/**************** Debug utilities *********************/

function propNames( obj )
{
    if( !obj ) return "obj is undefined";
    var s = "/*" + obj.toString() + ":*/\n{";
    var comma = false;
    for( p in obj ) {
        if( comma ) s += ",";
        comma = true;
        s += "\n\t" + p + "=" + obj[p];
    }
    s += "\n}";
    return s;
}

function inspect( obj ) 
{
    alert( propNames( obj ) );
}
